/* 
 *  File:  EchoServer.java
 * 
 *  This server waits for a connection, accepts it, opens two
 *  character streams, and immediately writes a welcome message
 *  to the output stream.  Then it waits for client input, and
 *  as it receives input, it echos the input back to the client
 *  via the output stream.  EchoServer runs until the connection 
 *  is closed by the client or the client issues a "BYE" command.
 *
 *  This version of EchoServer optionally takes a port number
 *  on the command line.
 *
 *  Usage:    % java EchoServer [<port>]
 *  Example:  % java EchoServer 5555
 *
 *  Adapted from an example of the same name by
 *  Gary Cornell and Cay S. Horstmann, Core Java (Book/CD-ROM)
 *
 */

import java.io.*;
import java.net.Socket;
import java.net.ServerSocket;
import java.net.BindException;

class EchoServer
{  
   private static int port = 4882;  // default port

   public static void main( String[] args ) 
   {  
      checkArgs( args );

      try 
      {  
         // open a server socket:
         ServerSocket serverSocket = new ServerSocket( port );
         System.out.println( "EchoServer running on port " + port + "..." );

         // wait for a connect message from client:
         Socket socket = serverSocket.accept();

         // open IO streams to the client:
         BufferedReader in =
           new BufferedReader(
             new InputStreamReader( socket.getInputStream() ) );
         boolean autoflush;
         PrintWriter out =
           new PrintWriter( socket.getOutputStream(), autoflush = true );

         out.println( "Hello! Enter BYE to exit EchoServer." );

         // Echo lines back to the client until the client 
         // sends "BYE" message or closes the connection:
         while ( true )
         {  String str = in.readLine();
            if ( str == null ) break;
            if ( str.trim().toUpperCase().equals( "BYE" ) ) break;
            out.println( "EchoServer: " + str );
         }
         
         // close the streams and socket:
         in.close();
         out.close();
         socket.close();
      } 
      catch ( BindException be ) 
      {
         System.err.println( be );
         System.out.println( "Specify port on command line." );
      }
      catch ( IOException e ) 
      {
         System.err.println( e );
      } 

   } 

   private static void checkArgs( String[] args ) 
   {
      // determine the port:
      if ( args.length > 0 ) 
         { try 
           {
            port = Integer.parseInt( args[0] );
           } 
           catch ( NumberFormatException e ) 
           {
            System.err.println( e );
            System.out.println( "Ignoring command-line argument" );
           }
         }
   }

}
