/* 
 *  File:  PortInspector.java
 *
 *  This client application inspects a range of ports on an 
 *  Internet host by attempting to connect to each port.  
 *  If a connection is made, a server is running on that port 
 *  (that is, the port is "used").  Note:  A refused connection 
 *  does not guarantee the port is available.
 *
 *  Usage:    % java PortInspector [<hostname> [<minPort> [<maxPort>]]]
 *  Example:  % java PortInspector osprey7.npac.syr.edu 5000 5999 | more
 * 
 *  Note:  Public ports are in the range 1024--65535.
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *
 */

import java.net.Socket;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.io.IOException;

class PortInspector {  

  // default host:
  private static String hostname = "osprey7.npac.syr.edu";
  private static InetAddress host;

  // by default, inspect the first few public ports:
  private static int minPort = 1024;
  private static int maxPort = minPort + 20;

  public static void main( String[] args ) {  

    checkArgs( args );
    
    // print hostname and IP address:
    System.out.print( "Host: " + hostname );
    System.out.println( " (" + host.toString() + ")" );
    
    Socket socket;

    for ( int port = minPort; port <= maxPort; port++ ) {
      try {
        socket = new Socket( host, port );
        System.out.println( "Port " + port + ":  used" );
        socket.close();
      } catch ( IOException e ) {
        System.out.println( "Port " + port + ":  " + e.getMessage() );
      }
    }
    
    System.exit(0);  // normal exit

  } 
  
  private static void checkArgs( String[] args ) {
  
    // check for too many arguments:
    if ( args.length > 3 ) {
      System.err.println( "Wrong number of arguments: " + args.length );
      System.exit( 1 );
    }
  
    // check for hostname argument:
    if ( args.length > 0 ) hostname = args[0];
    
    checkHost();
    
    // determine port range:
    if ( args.length > 1 ) {
      try {
        maxPort = minPort = Integer.parseInt( args[1] );
        if ( args.length == 3 ) {
          try {
            maxPort = Integer.parseInt( args[2] );
            // swap ports, if necessary:
            if ( maxPort < minPort ) {
              int temp = maxPort;
              maxPort = minPort;
              minPort = temp;
            }
          } catch ( NumberFormatException e ) {
            System.err.println( e );
            System.out.println( "Bad argument: " + args[2] );
          }
        }
      } catch ( NumberFormatException e ) {
        System.err.println( e );
        System.out.println( "Bad argument: " + args[1] );
        System.out.println( "Ignoring remaining command-line arguments" );
      }
    }

  }
  
  private static void checkHost() {
  
    try {
      host = InetAddress.getByName( hostname );
    } catch ( UnknownHostException e ) {
      System.err.println( e );
      System.exit( 2 );
    }
  
  }

}
