/*
 *  File:  ThreadedEchoServer.java
 * 
 *  This server is an extension of EchoServer 
 *  (see EchoServer.java) to accept connections from any
 *  number of clients.
 *
 *  This version of ThreadedEchoServer optionally takes a port
 *  number on the command line.
 *
 *  Usage:    % java ThreadedEchoServer [<port>]
 *  Example:  % java ThreadedEchoServer 5544
 *
 *  Adapted from an example of the same name by
 *  Gary Cornell and Cay S. Horstmann, Core Java (Book/CD-ROM)
 *
 */

import java.io.*;
import java.net.Socket;
import java.net.ServerSocket;
import java.net.BindException;

class ThreadedEchoServer
{
   private static int port = 7776;  // default port

   private static Socket ilk = null, second = null;

   public static void main( String[] args ) 
   {  
      checkArgs( args );
         
      try 
      {  
         // open a server socket:
         ServerSocket serversocket = new ServerSocket( port );

         while ( true )
         {  // wait for a connect message from client:
            Socket socket = serversocket.accept();
            // create a thread to open sockets to that client and
            // handle transaction

	    if ( ilk == null ) {
		ilk = socket;
	    } else {
		second = socket;
	    	(new ThreadedEchoHandler( ilk, second )).start();
		ilk = second = null;
	    }	
         }   
      }
      catch ( BindException be ) 
      {
         System.err.println( be );
         System.out.println( "Specify port on command line." );
      }
      catch ( IOException e ) 
      {
         System.err.println( e );
      } 

   } 

   private static void checkArgs( String[] args ) 
{
      // determine the port, if any:
      if ( args.length > 0 ) 
      { try 
        {
            port = Integer.parseInt( args[0] );
         } 
        catch ( NumberFormatException e ) 
        {
            System.err.println( e );
            System.out.println( "Ignoring bad command-line argument" );
         }
      }
   }
   
}

class ThreadedEchoHandler extends Thread
{
   // counts number of clients over all threads
   private static int numClients = 0;

   private Socket socket1, socket2;
   private int id;

   private ThreadedEchoHandler partner = null;
      
   ThreadedEchoHandler( Socket s1, Socket s2 ) 
   { socket1 = s1;
     socket2 = s2; 
     id = ++numClients; }
   
   public void run()
   {  try 
      {  

         BufferedReader in1 =
           new BufferedReader(
             new InputStreamReader( socket1.getInputStream() ) );
         boolean autoflush;
         PrintWriter out1 =
           new PrintWriter( socket1.getOutputStream(), autoflush = true );

         out1.println( "Hello client #" + id + "!  " );

         BufferedReader in2 =
           new BufferedReader(
             new InputStreamReader( socket2.getInputStream() ) );
         //boolean autoflush;
         PrintWriter out2 =
           new PrintWriter( socket2.getOutputStream(), autoflush = true );

         out2.println( "Hello client #" + id + "!  " );

         // Echo lines back to the client until the client 
         // closes the connection:
         while ( true )
         {  
            String str1 = in1.readLine();
            if ( str1 == null ) break;
            String str2 = in2.readLine();
            if ( str2 == null ) break;
            out1.println( "Echo # " + id +": " + str2 );
            out2.println( "Echo # " + id +": " + str1 );
         }
            
         // close the streams and socket:
         in1.close();
         out1.close();
         in2.close();
         out2.close();
         closeConnection();

      }
      catch ( IOException e ) 
      {
         System.err.println( e );
         closeConnection();
      } 
   } 

  private void closeConnection() 
  {
    try 
    { 
   socket1.close();
   socket2.close();
    } 
    catch ( IOException e ) 
   { System.err.println( e );
    }
    
  }
  
}
