/*
 *  File:  ThreadedEchoServer.java
 * 
 *  This server is an extension of EchoServer 
 *  (see EchoServer.java) to accept connections from any
 *  number of clients.
 *
 *  This version of ThreadedEchoServer optionally takes a port
 *  number on the command line.
 *
 *  Usage:    % java ThreadedEchoServer [<port>]
 *  Example:  % java ThreadedEchoServer 5544
 *
 *  Adapted from an example of the same name by
 *  Gary Cornell and Cay S. Horstmann, Core Java (Book/CD-ROM)
 *
 */

import java.io.*;
import java.net.Socket;
import java.net.ServerSocket;
import java.net.BindException;
import java.util.Vector;

class ThreadedEchoServer
{
   private static int port = 7776;  // default port

   private static ThreadedEchoHandler partner;

   public static void main( String[] args ) 
   {  
      checkArgs( args );
         
      try 
      {  
         // open a server socket:
         ServerSocket serversocket = new ServerSocket( port );

         while ( true )
         {  // wait for a connect message from client:
            Socket socket = serversocket.accept();
            // create a thread to open sockets to that client and
            // handle transaction

	    ThreadedEchoHandler newClient = new ThreadedEchoHandler( socket );
	    if ( partner == null ) {
	        partner = newClient;
	    } else {
	        partner.setPartner( newClient );
		newClient.setPartner( partner );
		partner = null;
	    }	
	    newClient.start();
         }   
      }
      catch ( BindException be ) 
      {
         System.err.println( be );
         System.out.println( "Specify port on command line." );
      }
      catch ( IOException e ) 
      {
         System.err.println( e );
      } 

   } 

   private static void checkArgs( String[] args ) 
{
      // determine the port, if any:
      if ( args.length > 0 ) 
      { try 
        {
            port = Integer.parseInt( args[0] );
         } 
        catch ( NumberFormatException e ) 
        {
            System.err.println( e );
            System.out.println( "Ignoring bad command-line argument" );
         }
      }
   }
   
}

class ThreadedEchoHandler extends Thread
{
   // counts number of clients over all threads
   private static int numClients = 0;

   private Socket socket;
   private int id;

  // chat only between two parties
   private ThreadedEchoHandler partner = null;
      
  // collect messages in a list so that other server can access them 
   private Vector messageList = new Vector();

   ThreadedEchoHandler( Socket s ) 
   { socket = s;
     id = ++numClients; 
   }
   
  public int ID() { return id; }

   public void setPartner( ThreadedEchoHandler p ) 
   { partner = p; }

  public boolean hasMessages() {
    return messageList.size() > 0;
  }
 
  public String getMessages() {
    String result = "";

    for(; messageList.size() > 0; messageList.remove( 0 ) ) 
      result += (String)(messageList.elementAt( 0 )) + "\n";

    return result;
  }

   public void run()
   {  try 
      {  

         BufferedReader in =
           new BufferedReader(
             new InputStreamReader( socket.getInputStream() ) );
         boolean autoflush;
         PrintWriter out =
           new PrintWriter( socket.getOutputStream(), autoflush = true );

         out.println( "Hello client #" + id + "! Waiting for your partner... " );

	while( partner == null ) {}

         // Echo lines back to the client until the client 
         // closes the connection:
         while ( true )
         {  
            String str = in.readLine();
            if ( str == null ) break;

	    // add messages to a queue
	    messageList.add( str );

	    if( partner.hasMessages() )
	      out.println( "# " + partner.ID() +": " + partner.getMessages() );
         }
            
         // close the streams and socket:
         in.close();
         out.close();
         closeConnection();

      }
      catch ( IOException e ) 
      {
         System.err.println( e );
         closeConnection();
      } 
   } 

  private void closeConnection() 
  {
    try 
    { 
   socket.close();
    } 
    catch ( IOException e ) 
   { System.err.println( e );
    }
    
  }
  
}
